/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDI_CORE_H

#define _GANDI_CORE_H	1

#include <stdlib.h>
#include <panel.h>
#include <limits.h>

#ifndef __USE_GNU
#	define __USE_GNU	1
#endif

#ifndef __USE_BSD
#	define __USE_BSD	1
#endif

extern short GANDI_actbord_pair;
extern attr_t GANDI_actbord_attr;
extern short GANDI_inactbord_pair;
extern attr_t GANDI_inactbord_attr;

//whether registrate widgets
extern bool GANDI_registrate;
//whether unregistrate widgets
extern bool GANDI_unregistrate;

typedef struct _GANDIW_widget GANDIW_widget;

//number of registered widgets
extern size_t GANDI_widgets;
//registered widgets
extern GANDIW_widget **GANDI_widget;
//active widget
extern size_t GANDI_actwidget;

#include "gandi_driver.h"
#include "gandi_timer.h"

struct _GANDIW_widget {
	void *			self;		//widget itself
	PANEL *			border;		//border window
	PANEL **		panlist;	//widget windows
	void *			parent;		//parent
	void **			child;		//children
	bool			live;		//wether it can be focused
	void *			resize_o;	//widget to resize
	bool			movable;	//move allowing switch
	void **			move_o;		//widgets to move
	int			rows_min;	//min lines number
	int			cols_min;	//min columns number
	int			rows_max;	//max lines number
	int			cols_max;	//max columns number
	bool			allow_border;
	bool			visible;	//whether it is visible
	GANDIT_key_alias **	key_alias;	//array of key aliases
	GANDIT_keymap **	keymap_sys;	//system key map
	GANDIT_keymap **	keymap;		//key map
	GANDIT_timer *		timer;		//timer structure
	GANDIact_actions	act;		//event handlers
	void *			userptr;	//one can store anything here
};

//creates color pairs
extern void GANDI_initpairs();
//initializes ncurses and applies standart properties to screen
//returns OK or ERR
extern int GANDI_init();
//standartizes window by applying some properties to it
extern void GANDI_stndrwin(WINDOW *w);
//frees memory allocated for widget
extern void GANDI_kill_emptywidget(void *ws);
//frees memory, allocated for widget's children; calls kill action handler
extern void GANDI_kill_widget(void *w);
//standartizes widget by applying some properties to it
extern void GANDI_stndrwidget(GANDIW_widget *w);
//makes an empty widget
extern void *GANDI_make_emptywidget(size_t size, int nlines, int ncols, int by,		int bx, void *parent, size_t wnum, void *dfl);
//makes a standart window with assigned parametres
extern PANEL *GANDI_make_stndrpan(int nlines, int ncols, int by, int bx,		attr_t attr, short pair);
//moves only assigned widget
extern int GANDI_mvonly_widget(void *w, int by, int bx);
//moves widget and all its children
extern int GANDI_mvwidget(void *w, int by, int bx);
#define GANDI_mvwidget_border(w, by, bx)						GANDI_mvwidget( (w),									(by) + GANDI_getborder(w), (bx) + GANDI_getborder(w) )
//resizes widget
extern int GANDI_resize_widget(void *w, int nlines, int ncols);
#define GANDI_resize_widget_border(w, nlines, ncols)					GANDI_resize_widget( (w),								(nlines) - GANDI_getborder(w) * 2,						(ncols) - GANDI_getborder(w) * 2)
//copies widget to the virtual screen
extern void GANDI_touchwidget_a(void *w, bool a);
extern void GANDI_touchwidget_ar(void *w, bool a);
extern bool GANDI_isancestor(void *p, void *c);
#define GANDI_touchwidget(w)								GANDI_touchwidget_a((w), (GANDI_active) ?						(GANDI_isancestor((w), GANDI_active->self) ||						GANDI_vtw(w) == GANDI_active) : FALSE)
//registers widget in system
extern int GANDI_registrate_widget(GANDIW_widget *w);
//unregisters widget in system
extern int GANDI_unregistrate_widget(GANDIW_widget *w);
//applies app to each panel of w
extern void GANDI_appwidget_panel(void *w, int(* app)(PANEL *a));
//applies app to each panel of w and its children
extern void GANDI_appwidgets_panel(void *w, int(* app)(PANEL *a));
//activates defined widget
extern size_t GANDI_activatewidget(void *ws);
//activates widget assigned by number
extern GANDIW_widget *GANDI_activateNwidget(size_t n);
//activates the next (or prev) widget
GANDIW_widget *GANDI_activate(bool next);
#define GANDI_activatenext()	GANDI_activate(TRUE)
#define GANDI_activateprev()	GANDI_activate(FALSE)
//returns the active widget's CORE
#define GANDI_getactive_widget()	( GANDI_widget[GANDI_actwidget] )
#define GANDI_active	GANDI_getactive_widget()
//returns number of widget's children
extern int GANDI_count_child(void *w);
//whether child really belongs to parent
extern bool GANDI_ischild(void *parent, void *child);
//adds one child to parent and returns ERR or OK
extern int GANDI_addchild(void *parent, void *child);
//removes one child from parent and returns ERR or OK
extern int GANDI_remchild(void *parent, void *child);
//counts number of pointers in list
extern size_t GANDI_countlist(void ***list);
//frees pointer list
extern void ***GANDI_freelist(void ***list);
//adds memb to the end of list
extern int GANDI_inclist(void ***list, void *memb);
extern int GANDI_inclist_q(void ***list, void *memb, size_t c);
//removes memb from list
extern int GANDI_declist(void ***list, void *memb);
//inserts memb before index
extern int GANDI_inslist(void ***list, void *memb, size_t index);
//returns members' index within list or NULL's index
extern size_t GANDI_listind(void ***list, void *memb);
//returns pointer to the list's member == memb or NULL
extern void **GANDI_listmemb(void ***list, void *memb);
extern void **GANDI_duplist_q(void ***list, size_t msize, size_t count);
#define GANDI_duplist(l,msize)	GANDI_duplist_q((l),(msize),GANDI_countlist(l))
//resizes panel
extern int GANDI_resize_panel(PANEL *p, int nlines, int ncols);
extern void GANDI_del_panel(PANEL *p);
//routines for move_o
#define GANDI_is_move_o(w,subj)								( *GANDI_listmemb( (void ***)&GANDI_vtw(w)->move_o, (subj) ) != NULL )
#define GANDI_rem_move_o(w,subj)							GANDI_declist( (void ***)&GANDI_vtw(w)->move_o, (subj) )
#define GANDI_add_move_o(w,subj)							( GANDI_is_move_o( (w), (subj) ) ? OK :							GANDI_inclist( (void ***)&GANDI_vtw(w)->move_o, (subj) ) )
int GANDI_clr_move_o(void *w);

//extracts a core from widget-pointer
#define GANDI_vtw(w) ( ((GANDIW_widget **)(w))[0] )
#define GANDI_wtv(w) ( ((void **)(w))[0] )
//routines for window control
#define GANDI_getbegyx(w, y, x) {							getbegyx(panel_window(GANDI_vtw(w)->border), (y), (x));				if (GANDI_vtw(w)->allow_border) { (y)++; (x)++; } }
#define GANDI_getbegy(w)								( getbegy( panel_window(GANDI_vtw(w)->border) ) +				(GANDI_vtw(w)->allow_border ? 1 : 0) )
#define GANDI_getbegx(w)								( getbegx( panel_window(GANDI_vtw(w)->border) ) +				(GANDI_vtw(w)->allow_border ? 1 : 0) )
#define GANDI_getmaxyx(w, y, x) {							getmaxyx(panel_window(GANDI_vtw(w)->border), (y), (x));				if (GANDI_vtw(w)->allow_border) { (y) -= 2; (x) -= 2; } }
#define GANDI_getmaxy(w)								( getmaxy( panel_window(GANDI_vtw(w)->border) ) -				(GANDI_vtw(w)->allow_border ? 2 : 0) )
#define GANDI_getmaxx(w)								( getmaxx( panel_window(GANDI_vtw(w)->border) ) -				(GANDI_vtw(w)->allow_border ? 2 : 0) )
#define GANDI_getbegyx_border(w, y, x)							getbegyx(panel_window(GANDI_vtw(w)->border), (y), (x))
#define GANDI_getbegy_border(w) getbegy( panel_window(GANDI_vtw(w)->border) )
#define GANDI_getbegx_border(w) getbegx( panel_window(GANDI_vtw(w)->border) )
#define GANDI_getmaxyx_border(w, y, x)							getmaxyx(panel_window(GANDI_vtw(w)->border), (y), (x))
#define GANDI_getmaxy_border(w) getmaxy( panel_window(GANDI_vtw(w)->border) )
#define GANDI_getmaxx_border(w) getmaxx( panel_window(GANDI_vtw(w)->border) )
#define GANDI_getborder(w)								((GANDI_getmaxy_border(w) - GANDI_getmaxy(w)) ? 1 : 0)
#define GANDI_get_Npanel(w,index)	( GANDI_vtw(w)->panlist[index] )
#define GANDI_get_Nwindow(w,index)							( panel_window(GANDI_vtw(w)->panlist[index]) )
#define GANDI_get_Nchild(w,index)	( GANDI_vtw(w)->child[index] )
//routines for setting widget properties
#define GANDI_setwidget_live(w,value)	( GANDI_vtw(w)->live = (value) )
#define GANDI_setwidget_movable(w,value)						( GANDI_vtw(w)->movable = (value) )
#define GANDI_setwidget_resize_o(w,value)						( GANDI_vtw(w)->resize_o = (value) )
#define GANDI_setwidget_resizable(w,value)						( GANDI_vtw(w)->resize_o = ((value) ? (w) : NULL) )
#define GANDI_setwidget_rows_min(w,val)							( (GANDI_getmaxy(w) < (val)) ? -1 : (GANDI_vtw(w)->rows_min = (val)) )
#define GANDI_setwidget_cols_min(w,val)							( (GANDI_getmaxx(w) < (val)) ? -1 : (GANDI_vtw(w)->cols_min = (val)) )
#define GANDI_setwidget_rows_max(w,val)							( (GANDI_getmaxy(w) > (val)) ? -1 : (GANDI_vtw(w)->rows_max = (val)) )
#define GANDI_setwidget_cols_max(w,val)							( (GANDI_getmaxx(w) > (val)) ? -1 : (GANDI_vtw(w)->cols_max = (val)) )
#define GANDI_setwidget_allow_syskeys(w,value)						( GANDI_vtw(w)->allow_syskeys = (value) )
#define GANDI_setwidget_key_alias(w,val)						( GANDI_vtw(w)->key_alias = (val) )
#define GANDI_setwidget_userptr(w,val)	( GANDI_vtw(w)->userptr = (val) )
extern bool GANDI_setwidget_allow_border(void *w, int value);
extern bool GANDI_setwidget_visible(void *w, bool value);
//routines for getting widget properties
#define GANDI_getwidget_parent(w)		( GANDI_vtw(w)->parent )
#define GANDI_getwidget_live(w)			( GANDI_vtw(w)->live )
#define GANDI_getwidget_movable(w)		( GANDI_vtw(w)->movable )
#define GANDI_getwidget_resize_o(w)		( GANDI_vtw(w)->resize_o )
#define GANDI_getwidget_resizable(w)							( GANDI_vtw(w)->resize_o != NULL )
#define GANDI_getwidget_rows_min(w)		( GANDI_vtw(w)->rows_min )
#define GANDI_getwidget_cols_min(w)		( GANDI_vtw(w)->cols_min )
#define GANDI_getwidget_rows_max(w)		( GANDI_vtw(w)->rows_max )
#define GANDI_getwidget_cols_max(w)		( GANDI_vtw(w)->cols_max )
#define GANDI_getwidget_allow_syskeys(w)	( GANDI_vtw(w)->allow_syskeys )
#define GANDI_getwidget_allow_border(w)		( GANDI_vtw(w)->allow_border )
#define GANDI_getwidget_visible(w)		( GANDI_vtw(w)->visible )
extern bool GANDI_iswidget_hidden(void *w);
#define GANDI_getwidget_key_alias(w)		( GANDI_vtw(w)->key_alias )
#define GANDI_getwidget_userptr(w)		( GANDI_vtw(w)->userptr )
//updates screen
#define GANDI_update()		{ update_panels(); doupdate(); }
#define GANDI_fiti(i, a, b) ( ((i)<(a)) ? (a) : ( ((i)>(b))?(b):(i) ) )
#define GANDI_fitu(i) ( (size_t)GANDI_fiti((i), 0, UINT_MAX) )

#endif //gandi_core.h

