/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDI_DRIVER_H

#define _GANDI_DRIVER_H	1

#include "gandi_core.h"

#ifdef GANDI_ctrl
#undef GANDI_ctrl
#endif
#define GANDI_ctrl(k) ((k) - '`')

#ifdef GANDI_meta
#undef GANDI_meta
#endif
#define GANDI_meta(k) ((k) | 0x200)

//type for return value of drivers
typedef int GANDIT_drvret;

//after any event
extern const GANDIT_drvret GANDI_drvJUST;
//no live widgets left
extern const GANDIT_drvret GANDI_drvNONE;
//widget received key and no exceptions are called
extern const GANDIT_drvret GANDI_drvCONT;
//focus to the next widget
extern const GANDIT_drvret GANDI_drvNEXT;
//focus to the previous widget
extern const GANDIT_drvret GANDI_drvPREV;
//quit program
extern const GANDIT_drvret GANDI_drvQUIT;
//stop action driver
extern const GANDIT_drvret GANDI_drvSTOP;

extern GANDIT_drvret(* GANDI_super_driver)(int val);

typedef struct _GANDIT_key_alias {
	int key;
	int alias;
} GANDIT_key_alias;

//rules for GANDI_stencil_keyalias
extern const int GANDI_keyalias_SET;
extern const int GANDI_keyalias_ADD;
extern const int GANDI_keyalias_REM;

extern const GANDIT_key_alias GANDI_keyalias_vim[];

typedef struct _GANDIT_keymap {
	int key;
	int value;
} GANDIT_keymap;

//action types
typedef void(* GANDIact_pre_focus)(void *);
typedef void(* GANDIact_post_focus)(void *);
/* the first argument of all routines is a widget */
/* these return ERR or OK */
typedef int(* GANDIact_resize)(void *, int nlines, int ncols, int y, int x);
/*	the second - a pressed key;
	the third argument of a post_driver routines is a result of a driver
		routine;
	pre_ routine changes key, which will be sent to the driver,
		by it's return value;
	return value of a post_ routine replaces driver's return value;
	Functions follow in this exactly sequence. */
typedef int(* GANDIact_pre_driver)(void *, int);
typedef GANDIT_drvret(* GANDIact_driver)(void *, int);
typedef GANDIT_drvret(* GANDIact_post_driver)(void *, int, GANDIT_drvret);
typedef void(* GANDIact_kill)(void *);

typedef struct _GANDIact_actions {
	GANDIact_pre_focus	pre_focus;
	GANDIact_post_focus	post_focus;
	GANDIact_resize		resize;
	GANDIact_pre_driver	pre_driver;
	GANDIact_driver		driver;
	GANDIact_post_driver	post_driver;
	GANDIact_kill		kill;
} GANDIact_actions;

//
extern GANDIT_keymap **GANDI_keymap_sys_stencil;

extern int GANDI_keymap_init();
//Gets key from w or from stdscr when NULL's given. Defines meta keys.
extern int GANDI_getch(void *w);
//returns key's value, if w has keymap, and k otherwise
extern int GANDI_get_keymap(GANDIT_keymap ***m, int k);
extern GANDIT_keymap **GANDI_set_keymap(GANDIT_keymap ***m, int key, int val);
//copies GANDIT_keymap ***s to GANDIT_keymap ***m
extern GANDIT_keymap ***GANDI_cpy_keymap(GANDIT_keymap ***d,				GANDIT_keymap ***s);
//executes widget's action handlers
extern GANDIT_drvret GANDI_action_driver(int key);
//returns alias of k or k itself if no alias defined
extern int GANDI_get_keyalias(void *w, int k);
//defines key alias
extern int GANDI_set_keyalias(void *w, int key, int alias);
//unaliases key
extern int GANDI_rem_keyalias(void *w, int key);
//modify key aliases using specified rule and stencil
extern int GANDI_stencil_keyalias(void *w, const GANDIT_key_alias stencil[],			int rule);
//gives control to widgets, which demand for it by timer
extern bool GANDI_timer_driver();
//drives system until stat is reached
extern GANDIT_drvret GANDI_driver(GANDIT_drvret stat);

#define GANDI_getwidget_keymap(w)	( GANDI_vtw(w)->keymap )
#define GANDI_getwidget_keymap_sys(w)	( GANDI_vtw(w)->keymap_sys )
//routines for setting action handlers
#define GANDI_setact_pre_focus(w, f)	( GANDI_vtw(w)->act.pre_focus = (f) )
#define GANDI_setact_post_focus(w, f)	( GANDI_vtw(w)->act.post_focus = (f) )
#define GANDI_setact_resize(w, f)	( GANDI_vtw(w)->act.resize = (f) )
#define GANDI_setact_driver(w, f)	( GANDI_vtw(w)->act.driver = (f) )
#define GANDI_setact_pre_driver(w, f)	( GANDI_vtw(w)->act.pre_driver = (f) )
#define GANDI_setact_post_driver(w, f)	( GANDI_vtw(w)->act.post_driver = (f) )
#define GANDI_setact_kill(w, f)		( GANDI_vtw(w)->act.kill = (f) )
//routines for getting action handlers
#define GANDI_getact_pre_focus(w)	( GANDI_vtw(w)->act.pre_focus )
#define GANDI_getact_post_focus(w)	( GANDI_vtw(w)->act.post_focus )
#define GANDI_getact_resize(w)		( GANDI_vtw(w)->act.resize )
#define GANDI_getact_driver(w)		( GANDI_vtw(w)->act.driver )
#define GANDI_getact_pre_driver(w)	( GANDI_vtw(w)->act.pre_driver )
#define GANDI_getact_post_driver(w)	( GANDI_vtw(w)->act.post_driver )
#define GANDI_getact_kill(w)		( GANDI_vtw(w)->act.kill )

#endif //gandi_driver.h

