/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDIW_BUTTON_GROUP_H

#define _GANDIW_BUTTON_GROUP_H	1

#include "gandi_core.h"

typedef struct _GANDIW_button_group GANDIW_button_group;

struct _GANDIW_button_group {
	GANDIW_widget *		core;		//common for all widgets
	attr_t			win_attr;
	short			win_pair;
	attr_t			cursor_attr;
	short			cursor_pair;
	void **			button;		//all its buttons
	size_t			cursor;		//the cursored button
	bool			radio;		//reject multi check
	size_t			sel;		//the selected btn
};

extern GANDIW_button_group GANDIW_button_group_dfl;
extern GANDIW_widget GANDIW_button_group_dfl_core;

extern GANDIW_button_group *GANDIW_button_group_make(int nlines, int ncols,		int by, int bx, void *parent);
extern int GANDIW_button_group_resize(GANDIW_button_group *w, int nlines,		int ncols, int by, int bx);
extern void GANDIW_button_group_kill(GANDIW_button_group *w);
extern int GANDIW_button_group_add_btn(GANDIW_button_group *w, void *btn);
extern int GANDIW_button_group_rem_btn(GANDIW_button_group *w, void *btn);
extern void GANDIW_button_group_mv_cursor(GANDIW_button_group *w, bool down);
//refreshes window and if cur == TRUE - only current button, if cur == FALSE -
//only inactive buttons, if cur == -TRUE - all buttons
extern void GANDIW_button_group_refresh(GANDIW_button_group *w, short cur);
extern GANDIT_drvret GANDIW_button_group_ch_Nbtn(GANDIW_button_group *w,		size_t index);
extern int GANDIW_button_group_driver(GANDIW_button_group *w, int k);

#define GANDIW_button_group_vtw(w)	( (GANDIW_button_group *)(w) )
#define GANDIW_button_group_window(w)	GANDI_get_Nwindow((w), 0)	

#define GANDIW_button_group_get_Nbtn(w,index)						( GANDIW_button_group_vtw(w)->button[index] )

#define GANDIW_button_group_get_Ncapwid(w,index)					( GANDI_get_Nchild(GANDIW_button_group_vtw(w)->button[index], 0) )

#define GANDIW_button_group_get_curbtn(w)						GANDIW_button_group_get_Nbtn( (w), GANDIW_button_group_get_cursor(w) )

#define GANDIW_button_group_get_cursor(w)						( GANDIW_button_group_vtw(w)->cursor )
extern int GANDIW_button_group_set_cursor(GANDIW_button_group *w, int val);

#define GANDIW_button_group_get_radio(w)						( GANDIW_button_group_vtw(w)->radio )
#define GANDIW_button_group_set_radio(w,value)						( GANDIW_button_group_vtw(w)->radio = (value) )

#define GANDIW_button_group_set_attr_on(w,index)					wbkgd( GANDI_get_Nwindow(								GANDIW_button_group_get_Ncapwid((w),(index)), 0 ),				GANDIW_button_group_vtw(w)->cursor_attr |					COLOR_PAIR(GANDIW_button_group_vtw(w)->cursor_pair) )
#define GANDIW_button_group_set_attr_off(w,index)					wbkgd( GANDI_get_Nwindow(								GANDIW_button_group_get_Ncapwid((w),(index)), 0 ),				GANDIW_button_group_vtw(w)->win_attr |						COLOR_PAIR(GANDIW_button_group_vtw(w)->win_pair) )

#define GANDIW_button_group_count(w)							GANDI_countlist( (void ***)&(GANDIW_button_group_vtw(w)->button) )

extern attr_t GANDIW_button_group_set_win_attr(GANDIW_button_group *w,			attr_t val);
#define GANDIW_button_group_get_win_attr(w)						( GANDIW_button_group_vtw(w)->win_attr )

extern short GANDIW_button_group_set_win_pair(GANDIW_button_group *w,			short val);
#define GANDIW_button_group_get_win_pair(w)						( GANDIW_button_group_vtw(w)->win_pair )

extern attr_t GANDIW_button_group_set_cursor_attr(GANDIW_button_group *w,		attr_t val);
#define GANDIW_button_group_get_cursor_attr(w)						( GANDIW_button_group_vtw(w)->cursor_attr )

extern short GANDIW_button_group_set_cursor_pair(GANDIW_button_group *w,		short val);
#define GANDIW_button_group_get_cursor_pair(w)						( GANDIW_button_group_vtw(w)->cursor_pair )

#endif

