/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDIW_SCROLL_BOX_H

#define _GANDIW_SCROLL_BOX_H	1

#include "gandiw_scroll_bar.h"

typedef struct _GANDIW_scroll_box GANDIW_scroll_box;

struct _GANDIW_scroll_box {
	GANDIW_widget		*core;		//common for all widgets
	PANEL			*subj;		//panel in subjection
	GANDIW_scroll_bar	*vscr;		//vertical scroll bar
	GANDIW_scroll_bar	*hscr;		//horizontal scroll bar
	size_t			rows;		//lines in storage
	size_t			cols;		//columns in storage
	short			mask;		//mask scroll bars
	int			miny, minx;	//minimal size of subj panel
};

extern GANDIW_widget GANDIW_scroll_box_dfl_core;
extern GANDIW_scroll_box GANDIW_scroll_box_dfl;

extern GANDIW_scroll_box *GANDIW_scroll_box_make(int nlines, int ncols, int by,		int bx, void *parent, PANEL *subj);
extern int GANDIW_scroll_box_resize(GANDIW_scroll_box *w, int nlines, int ncols,	int by, int bx);
extern void GANDIW_scroll_box_kill(GANDIW_scroll_box *w);
extern void GANDIW_scroll_box_show(GANDIW_scroll_box *w);
extern void GANDIW_scroll_box_set_area(GANDIW_scroll_box *w, long long rows,		long long cols);
/* This function prepares widget for change mask; it returns ERR if val is
cannot be set now as mask; when it returns OK, it sets rows_min and cols_min
of widget as they must be with the supposed mask val. */
extern int GANDIW_scroll_box_calcmin(GANDIW_scroll_box *w, short val);
extern bool GANDIW_scroll_box_adjust(GANDIW_scroll_box *w, size_t y, size_t x);
extern GANDIT_drvret GANDIW_scroll_box_driver(GANDIW_scroll_box *w, int k);

#define GANDIW_scroll_box_vtw(w)	( (GANDIW_scroll_box *)w )

#define GANDIW_scroll_box_vscr(w)	( GANDIW_scroll_box_vtw(w)->vscr )
#define GANDIW_scroll_box_hscr(w)	( GANDIW_scroll_box_vtw(w)->hscr )

#define GANDIW_scroll_box_get_rows(w)	( GANDIW_scroll_box_vtw(w)->rows )
extern size_t GANDIW_scroll_box_set_rows(GANDIW_scroll_box *w, long long val);
#define GANDIW_scroll_box_get_cols(w)	( GANDIW_scroll_box_vtw(w)->cols )
extern size_t GANDIW_scroll_box_set_cols(GANDIW_scroll_box *w, long long val);

#define GANDIW_scroll_box_get_mask(w)	( GANDIW_scroll_box_vtw(w)->mask )
extern short GANDIW_scroll_box_set_mask(GANDIW_scroll_box *w, short val);

#define GANDIW_scroll_box_get_miny(w)	( GANDIW_scroll_box_vtw(w)->miny )
extern int GANDIW_scroll_box_set_miny(GANDIW_scroll_box *w, int val);

#define GANDIW_scroll_box_get_minx(w)	( GANDIW_scroll_box_vtw(w)->minx )
extern int GANDIW_scroll_box_set_minx(GANDIW_scroll_box *w, int val);

// Additional mask implements
#define GANDIW_scroll_box_mask(w, val)							GANDIW_scroll_box_set_mask(w, GANDIW_scroll_box_get_mask(w) | (val))

#define GANDIW_scroll_box_unmask(w, val)						( ( GANDIW_scroll_box_get_mask(w) & (val) ) ?					GANDIW_scroll_box_set_mask(w, GANDIW_scroll_box_get_mask(w) - (val)) :		GANDIW_scroll_box_get_mask(w) )

#endif

